#include <cnoid/SimpleController>
#include <vector>
#include <iostream>
#include <Eigen/Dense>

using namespace cnoid;

class closed_link_controller : public SimpleController {
  Link* joint0;
  double dt;
  double t = 0;
  double tf = 10;

private:
  double q0;
  Eigen::Vector3d des, par;
  Eigen::Vector4d a;
  
public:
  virtual bool initialize(SimpleControllerIO *io) override;
  virtual bool control() override;
  Eigen::Vector3d makeSpline3(double, double, double, double, double, double);
};

bool closed_link_controller::initialize(SimpleControllerIO* io){
  dt = io->timeStep();
  joint0 = io->body()->joint(0);
  joint0->setActuationMode(Link::JointAngle);
  io->enableIO(joint0);
  q0 = joint0->q();
  
  return true;
}

bool closed_link_controller::control(){
  
  double q = joint0->q();
  if(t < tf)
    des = makeSpline3(t, tf, q0, 0, 10 * M_PI, 0);
  joint0->q_target() = des(0);

  t = t + dt;

  return true;
}

Eigen::Vector3d closed_link_controller::makeSpline3(double t, double tf, double xi0, double dxi0, double xif, double dxif){
  a <<
    xi0,
    dxi0,
    (-3.*xi0 - 2.*dxi0*tf + 3.*xif - dxif*tf) / (tf*tf),
    (2.*xi0 + dxi0*tf - 2.*xif + dxif*tf) / (tf*tf*tf);

  par(0) = a(0) + a(1)*t + a(2)*t*t + a(3)*t*t*t;
  par(1) = a(1) + 2.*a(2)*t + 3.*a(3)*t*t;
  par(2) = 2.*a(2) + 6.*a(3)*t;

  return par;
}

CNOID_IMPLEMENT_SIMPLE_CONTROLLER_FACTORY(closed_link_controller)